#include "netrun.h"
/* va_args */
#include <stdarg.h>
/* usleep */
#include <unistd.h>
/* *printf, stderr */
#include <stdio.h>
/* time_t time(), ctime() */ /* for log() */
#include <time.h>

/*
 * Consistent menu-mode menu pretty-printing. Still here to avoid changing a lot of code.
 */

/* Clears stuff for printing a menu. */
void menu_init(void)
{
    if (is_running())
        return;    

    pr_pick(menus);
    pr_blank();
    pr("\n\n");
}

/*
 * Prints a line to the menus window, to be later updated by menu_print.
 * Slightly different actions in netrunning mode.
 * Pretty prints - square brackets around key, indentation, extra carriage returns, etc.
 * menu("", "Buy which type of program?"); menu("A", "Anti-Program"); produces
 * something like:
 *
 *                 -*-  Buy which type of program?
 *
 *                 [A]  Anti-Program
 *
 * HACK Note that KEY cannot have special arguments - that's for the FMT bit.
 * HACK This means that one sees uses of sprintf in the main code, *drat*.
 * IDEA Such uses of sprintf are usually for numbers. Perhaps use a void data type and argument checker?
 */
void menu(char * key, char * fmt, ...)
{
    char buf[200];
    va_list args;
    char buf2[200];
    char buf3[200];

    if (is_running())
        return;    

    /* Hopefully this will handle zero-length strings. */
    buf[0] = '\0';

    /* Collect arguments. I hope to see no use of sprintf in the main code. */
    va_start(args, fmt);
    vsnprintf(buf, 200, fmt, args);
    va_end(args);

    /* Null key? It's a title. */
    if ((key != NULL) && (key[0] != '\0'))
        sprintf(buf3,"[%s]", key);
    else
        sprintf(buf3,"-*-");

    /*
     * Menus in non-running mode are indented by about sixteen spaces so the 
     * screen does not look too empty.
     * Program lists just get the indent.
     */    
    /* Compose the menu line. */
    if (!is_running())
    {
        sprintf(buf2,"        %s  %s\n",
            buf3,
            buf);
    }
    else
        sprintf(buf2,"%s %s\n",
            buf3,
            buf);
        
    /* Print. */
    pr(buf2);

    /* All done. */
    return;
}

/*
 * Print menu_buf to appropriate screen.
 */
void menu_print(void)
{
    pr_paint();

    /* All done. */
    return;
}


/* Physically wipes clear the errs line. Called by input* and event (to clear up the line after morewait()). */
void err_blank(void)
{
    /* HACK Dirty, dirty hack. Ugh. */
    WINDOW * tmp;
    tmp = pr_window;
    pr_pick(errs);
    pr_blank();
    pr_paint();
    pr_window = tmp;
}

/*
 * Prints a message to the screen (you have insufficient credits) that is highly transient, usually wiped by the next keystroke.
 */

void err(char * fmt, ...)
{
    char buf[200];
    va_list args;

    /* Collect args */    
    va_start(args, fmt);
    vsnprintf(buf, 200, fmt, args);
    va_end(args);
    
    /* Print it. */
    pr_pick(errs);
    pr_blank();
    pr(buf);
    pr_paint();
}

/*
 * Prints a less transient message in the same place as err() - this calls morewait(), then wipes the message.
 */

void event(char * fmt, ...)
{
    char buf[200];
    char buf2[200];
    va_list args;

    if (errs == NULL)
        return;

    if (is_running())
        return;    

    /* Gather question args */    
    va_start(args, fmt);
    vsnprintf(buf, 200, fmt, args);
    va_end(args);

    /* Clear line, print the event (" * " marks as an event rather than an error). */
    pr_pick(errs);
    pr_blank();
    pr(" * %s", buf);
    pr_paint();
    
    /* Wait for keypress (with twiddler!) */
    morewait();
    /* Clear up the event. */
    err_blank();
}

/* Report a bug, both to screen and to log. */
void bug(char * fmt, ...)
{
    char buf[200];
    va_list args;
    
    va_start(args, fmt);
    vsnprintf( buf, 200, fmt, args);
    va_end(args);
   
    err(buf);
    if (pLog != NULL)
        log("BUG: %s", buf);
}    

/* Log a string into the game's logfile. */
void log(char * fmt, ...)
{
    char buf[200];
    va_list args;
    
    va_start(args, fmt);
    vsnprintf( buf, 200, fmt, args);
    va_end(args);

    if (pLog != NULL)
    {    
        static time_t t;
        char * strtime;
    
        t = time(NULL);
        strtime = ctime(&t);
        strtime[strlen(strtime) - 1] = '\0'; /* Borrowed from Rom */
        fprintf(pLog, "%s: %s\n", strtime, buf);
        fflush(pLog); /* I'd like to be able to read it _now_, thanks. */
    }   
}

/* This _does_ have a use. See below. */
char desc_buf[10000];

void desc(char * fmt, ...)
{
    char buf[200];
    va_list args;

    if (descs == NULL)
        return;

    va_start(args, fmt);
    vsnprintf(buf, 200, fmt, args);
    va_end(args);

    if (is_running())
    {
        strcat(desc_buf, buf);

        /* HACK There's got to be a _cleaner_ way of doing this. */
        if(strchr(desc_buf,'\n'))
        {
            wprintw(descs, word_wrap(" * ", "   ", desc_buf, 79));
            wrefresh(descs);
            desc_buf[0] = '\0';
        }
    }
    else
        bug("desc: not 'running");
}

/* Clear the descs window. */
void desc_clear(void)
{
    if (descs == NULL)
        return;

    wmove(descs,0,0);
    werase(descs);
}

/* Used to print the stat line in non-running mode. */
void stat(char * fmt, ...)
{
    char buf[200];
    va_list args;

    if (is_running())
        return;
    
    if (player_stats == NULL)
        return;
    
    va_start(args, fmt);
    vsnprintf(buf, 200, fmt, args);
    va_end(args);

    werase(player_stats);
    wmove(player_stats, 0, 0);
    waddstr(player_stats,buf);
    wnoutrefresh(player_stats);
    
    if (!is_running())
        doupdate();
}

void init_output(void)
{
    /* Init ncurses */
    initscr();
    cbreak();
    noecho();
    
    title = NULL;
    display = NULL;
    programs = NULL;
    menus = NULL;
    descs = NULL;
    errs = NULL;
    inputs = NULL;
    bottom = NULL;
    player_stats = NULL;

    /* Initial windows */
    /* HACK assuming 80x25 -> no SIGWINCH handler yet. */    
    title = newwin(1,80,0,0);    
    menus = newwin(21,80,1,0);    
    player_stats = newwin(1,80,21,0);   
    errs = newwin(1,80,22,0);
    inputs = newwin(1,80,23,0);
    /* HACK Even bigger hack... */
    if (LINES > 24)
        bottom = newwin(1,80,24,0);

    /* Titles */
    waddstr(title,cline("- --- * Netrun " VERSION " * --- -","  --- -","- ---  ","=", COLS));
    wrefresh(title);

    if (LINES > 24)
    {    
        waddstr(bottom,cline("","+","+","-", COLS));
        wrefresh(bottom);
    }
    
    /* init log */
    pLog = fopen("netrun.log", "a"); /* HACK Specify in config, etc. */

    if (pLog == NULL)
    {
        bug("init_log: Can't open the log!");
        exit_properly();
        return;
    }
    
    log(" ** Log opened. ** ");
}

/* Close output. */
void stop_output(void)
{
    /* stop log */
    log(" ** Log closed. ** ");
    fclose(pLog);
    pLog = NULL;

    endwin();        
}

/*
 * The 'character-sparse' pr interface. Call it trying to fit it in 80 columns.
 */

/* Pick a window to work with. */
void pr_pick(WINDOW * win)
{
    if (win == NULL)
        return;
    
    pr_window = win;
    
}

/* Wipe the current window. */
void pr_blank(void)
{
    werase(pr_window);
    wmove(pr_window, 0, 0);
}

/* Move the cursor in the current window to a location. */
void prm(int y, int x)
{
    if (pr_window == NULL)
        return;

    wmove(pr_window, y, x);
}

/* Move the cursor in the current window by a given amount. */
void prd(int y2, int x2)
{
   int x, y;
  
    if (pr_window == NULL)
        return;

   getyx(pr_window, y, x);
   
   wmove(pr_window, y + y2, x + x2);   
}

/* Print a string to the current window. */
void pr(char * fmt, ...)
{
    char buf[200];
    va_list args;

    if (pr_window == NULL)
        return;

    /* Collect args */
    va_start(args, fmt);
    vsnprintf( buf, 200, fmt, args);
    va_end(args);

    if (pr_window == NULL)
        return;

    waddstr(pr_window, buf);
}

/* Print an 'extended' character to the current window. */
void prc(chtype ch)
{
    if (pr_window == NULL)
        return;

    waddch(pr_window, ch);
}

/* Repeat a number of extended characters to the current window. Useful for drawing lines. */
void rch(int count, chtype ch)
{
    int i;
    
    for (i = 0; i < count; i++)
    {
        prc(ch);
    }
}

/* Print the window to the screen (wnoutrefresh()), and forget which window we're working on. */
void pr_print(void)
{
    if (pr_window == NULL)
        return;

    wnoutrefresh(pr_window);
    pr_window = NULL;
}

/* Paint the window to the screen (doupdate()), and forget which window we're working on. */
void pr_paint(void)
{
    if (pr_window != NULL)
    {
        pr_print();
    }
    
    doupdate();
}









